/*
 * Copyright 2004-2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.hdiv.web.servlet.tags.form;

import java.util.StringTokenizer;

import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.hdiv.dataComposer.IDataComposer;
import org.hdiv.util.HDIVUtil;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.tags.form.FormTag;
import org.springframework.web.servlet.tags.form.TagWriter;
import org.springframework.web.util.HtmlUtils;

/**
 * Databinding-aware JSP tag for rendering an HTML '<code>form</code>' whose
 * inner elements are bound to properties on a
 * {@link #setCommandName command object}.
 * 
 * <p>
 * Users should place the command object into the
 * {@link org.springframework.web.servlet.ModelAndView} when populating the data
 * for their view. The name of this command object can be configured using the
 * {@link #setCommandName commandName} property.
 * 
 * <p>
 * The default value for the {@link #setCommandName commandName} property is '<code>command</code>'
 * which corresponds to the default name when using the
 * {@link org.springframework.web.servlet.mvc.SimpleFormController}.
 * 
 * <p>
 * Inner tags can access the name of the command object via the
 * {@link javax.servlet.jsp.PageContext}. The attribute name is defined in
 * {@link #COMMAND_NAME_VARIABLE_NAME}.
 * 
 * @author Gorka Vicente
 * @since HDIV 2.0
 * @see org.springframework.web.servlet.tags.form.FormTag
 */
public class FormTagHDIV extends FormTag {

	private TagWriter tagWriter;

	private static final String DATA_COMPOSER = "dataComposer";

	private IDataComposer dataComposer;
	

	/**
	 * Writes the opening part of the block '<code>form</code>' tag and
	 * exposes the command name in the {@link javax.servlet.jsp.PageContext}.
	 * 
	 * @param tagWriter the {@link TagWriter} to which the form content is to be
	 *            written
	 * @return {@link javax.servlet.jsp.tagext.Tag#EVAL_BODY_INCLUDE}
	 */
	protected int writeTagContent(TagWriter tagWriter) throws JspException {

		this.tagWriter = tagWriter;
		return super.writeTagContent(tagWriter);
	}

	/**
	 * Resolve the value of the '<code>action</code>' attribute.
	 * <p>
	 * If the user configured an '<code>action</code>' value then the result
	 * of evaluating this value is used. Otherwise, the
	 * {@link org.springframework.web.servlet.support.RequestContext#getRequestUri() originating URI}
	 * is used.
	 * 
	 * @return the value that is to be used for the '<code>action</code>'
	 *         attribute
	 */
	protected String resolveAction() throws JspException {

		dataComposer = (IDataComposer) this.pageContext.getRequest().getAttribute(DATA_COMPOSER);
		
		String action = getAction();
		if (StringUtils.hasText(action)) {
											
			String displayString = getDisplayString(evaluate(ACTION_ATTRIBUTE, action));
			
			String beginAction = displayString;
			if (displayString.contains("?")) {
				beginAction = displayString.substring(0, displayString.indexOf("?"));
			}
			dataComposer.beginRequest(HDIVUtil.getActionMappingName(beginAction));
			
			if (displayString.contains("?")) {
				String encodedParams = this.composeQueryString(displayString.substring(displayString.indexOf("?") + 1));
				displayString = beginAction + "?" + encodedParams;
			}
			
			return displayString;

		} else { // action == null
			String requestUri = getRequestContext().getRequestUri();
			ServletResponse response = this.pageContext.getResponse();

			if (response instanceof HttpServletResponse) {

				requestUri = ((HttpServletResponse) response).encodeURL(requestUri);
				
				dataComposer.beginRequest(HDIVUtil.getActionMappingName(requestUri));

				String queryString = getRequestContext().getQueryString();
				if (StringUtils.hasText(queryString)) {

					queryString = this.composeQueryString(queryString);
					requestUri += "?" + HtmlUtils.htmlEscape(queryString);
				}
			}
			if (StringUtils.hasText(requestUri)) {
				return requestUri;
			} else {
				throw new IllegalArgumentException("Attribute 'action' is required. Attempted to resolve "
						+ "against current request URI but request URI was null");
			}
		}
	}

	/**
	 * Removes HDIV parameter from <code>queryString</code> and it composes
	 * other parameters.
	 * 
	 * @param queryString query string
	 * @return queryString without HDIV's parameter
	 */
	protected String composeQueryString(String queryString) {

		String token = null;
		StringBuffer result = new StringBuffer();
		
		StringTokenizer st = new StringTokenizer(queryString, "&");
		while (st.hasMoreTokens()) {

			token = st.nextToken();
			String param = token.substring(0, token.indexOf("="));

			if (!ignoreParameter(param)) {

				String originalValue = this.pageContext.getRequest().getParameter(param);
				String val = dataComposer.compose(param, originalValue, false);
								
				if (result.length() > 0) {
					result.append("&");
				}
				result.append(param + "=" + val);
			}
		}
		return result.toString();
	}

	/**
	 * @returns Returns true if parameter <code>param</code> must be ignored.
	 *          False otherwise.
	 */
	protected boolean ignoreParameter(String param) {
		
		String hdivParameter = (String) HDIVUtil.getHttpSession().getAttribute("HDIVParameter");
		return param.equalsIgnoreCase(hdivParameter);
	}

	/**
	 * Closes the '<code>form</code>' block tag and removes the command name
	 * from the {@link javax.servlet.jsp.PageContext}.
	 */
	public int doEndTag() throws JspException {
	
		addExtraParameters();
		return super.doEndTag();
	}
	
	/**
	 * Aade nuevos parmetros al formulario que no han sido definidos mediante
	 * tags de HTML.
	 */
	public void addExtraParameters() throws JspException {
		
		this.addHDIVParameter();
	}
	
	/**
	 * Adds HDIV state as parameter.
	 */
	protected void addHDIVParameter() throws JspException {
		
		String requestId = dataComposer.endRequest();

		if (requestId.length() > 0) {
			String hdivState = (String) HDIVUtil.getHttpSession().getAttribute("HDIVParameter");
			this.tagWriter.appendValue(this.generateHiddenTag(hdivState, requestId));
		}
	}	

	/**
	 * Renders an HTML <b>&lt;input&gt;</b> element of type hidden.
	 * 
	 * @param name hidden parameter name
	 * @param requestId request identification
	 * @return HTML <b>&lt;input&gt;</b> element of type hidden
	 */
	protected String generateHiddenTag(String name, String requestId) {

		StringBuffer hdivParameter = new StringBuffer();

		hdivParameter.append("<input type=\"hidden\"");
		renderAttribute(hdivParameter, "name", name);
		renderAttribute(hdivParameter, "value", requestId);
		hdivParameter.append(">\n");

		return hdivParameter.toString();
	}

	/**
	 * Prepares an attribute if the <code>value</code> is not null, appending
	 * it to the the given StringBuffer <code>result</code>.
	 * 
	 * @param result The StringBuffer that output will be appended to.
	 */
	private void renderAttribute(StringBuffer result, String name, String value) {

		if (value != null) {
			result.append(" ");
			result.append(name);
			result.append("=\"");
			result.append(value);
			result.append("\"");
		}
	}

	public IDataComposer getDataComposer() {
		return dataComposer;
	}
	
	/**
	 * @return the tag writer
	 */
	public TagWriter getTagWriter() {
		return tagWriter;
	}

}
